searchNodes=[{"doc":"Smee is a pragmatic library for handling SAML metadata with Elixir, Erlang or any other BEAM language.FeaturesDownload remote SAML metadata or load local files, with effective cachingManage and compare metadata files and individual entity metadataMDQ API (which can also emulate MDQ style lookups with aggregate files)A focus on streaming with reliable and surprisingly low memory usageFilter entity streams by various criteriaValidate XML signatures, automatically download and confirm signing certificatesTransform metadata using XSLT, or extract dataAccess XML using Erlang's Xmerl library (sweetened by SweetXML)Recombine entity streams into aggregates or other data formatsCan be used with applications or in simple .exs scriptsModulesThe top level Smee module contains a few simplified, top level functions better suited to simpler scripts. Other modules in Smee contain more tools for handling SAML metadata, such as:Smee.Source - define sources of metadataSmee.Metadata - functions for handling metadata aggregatesSmee.Entity - individual SAML entity definitionsSmee.Extract - processing metadata to extract informationSmee.Fetch - downloading metadata sourcesSmee.MDQ - functions for MDQ clients (and emulating MDQ clients)Smee.Filter - filtering streams of entity recordsSmee.Transform - processing and editing entity XMLSmee.Publish - Formatting and outputting metadata in various formatsSmee.Stats - Simple stats for entity streamsSmee.Lint - XML validation and reformatting","ref":"Smee.html","title":"Smee","type":"module"},{"doc":"Lists the IDs of every entity in the metadata.This version of the function can accept either a %Source{} or a %Metadata{} struct containing already-loaded Metadata.Exampleiex&gt; Smee.source(&quot;http://metadata.ukfederation.org.uk/ukfederation-metadata.xml&quot;) iex&gt; |&gt; Smee.entity_ids()","ref":"Smee.html#entity_ids/1","title":"Smee.entity_ids/1","type":"function"},{"doc":"Downloads a source of metadata (local or remote) and returns a %Metadata{} struct containing XML and information.Exampleiex&gt; &quot;http://metadata.ukfederation.org.uk/ukfederation-metadata.xml&quot; iex&gt; |&gt; Smee.source() iex&gt; |&gt; Smee.fetch!()","ref":"Smee.html#fetch!/1","title":"Smee.fetch!/1","type":"function"},{"doc":"Retrieves information for a single entity from an MDQ service (real or emulated) and returns an %Entity{} struct.This version of the function can accept either a %Source{} or a %Metadata{} struct containing already-loaded Metadata.Exampleiex&gt; &quot;http://mdq.ukfederation.org.uk/&quot; iex&gt; |&gt; Smee.source(type: :mdq) iex&gt; |&gt; Smee.lookup!(&quot;https://cern.ch/login&quot;) iex&gt; &quot;http://metadata.ukfederation.org.uk/ukfederation-metadata.xml&quot; iex&gt; |&gt; Smee.source(type: :aggregate) iex&gt; |&gt; Smee.lookup!(&quot;https://cern.ch/login&quot;)","ref":"Smee.html#lookup!/2","title":"Smee.lookup!/2","type":"function"},{"doc":"Defines a source of metadata Sources of metadata include online aggregate XML, local aggregate files, individual entities, and MDQ services. This function will only define sources of aggregate XML.Exampleiex&gt; Smee.source(&quot;http://metadata.ukfederation.org.uk/ukfederation-metadata.xml&quot;) iex&gt;","ref":"Smee.html#source/1","title":"Smee.source/1","type":"function"},{"doc":"Defines a source of metadata Sources of metadata include online aggregate XML, local aggregate files, individual entities, and MDQ services. This function allows a lot of customisation, particularly the type. Types are::aggregate (a file containing a collection of entityDescriptor fragments inside a entitiesDescriptor tag, as used by federations):single (a file with a single entityDescriptor, as used for individual metadata records) :mdq (an online MDQ service)URLs may be remote (http:// and https://) or local (file://). Local files can be specified as bare paths.See Smee.Source.new for full detailsExampleiex&gt; Smee.source(&quot;http://mdq.ukfederation.org.uk/&quot;, type: :mdq, retries: 1, label: &quot;UK MDQ Service&quot;) iex&gt; Smee.source(&quot;support/static/valid.xml&quot;, type: :single, retries: 1, label: &quot;My IdP&quot;)","ref":"Smee.html#source/2","title":"Smee.source/2","type":"function"},{"doc":"Streams all entities in the specified metadata or source.This version of the function can accept either a %Source{} or a %Metadata{} struct containing already-loaded Metadata.Exampleiex&gt; Smee.stream_entities(&quot;http://metadata.ukfederation.org.uk/ukfederation-metadata.xml&quot;) iex&gt; |&gt; Stream.take(1) iex&gt; |&gt; Enum.to_list","ref":"Smee.html#stream_entities/1","title":"Smee.stream_entities/1","type":"function"},{"doc":"Smee wraps up metadata for individual entities in %Entity{} structs, and the Smee.Entity module contains functions that may be useful when working with them.Many of the functions mirror those in the Smee.Metadata module - the same actions but on a smaller unit of XML.Like %Metadata{} structs the XML in entities can be compressed and decompressed, but unlike Metadata structs they have parsed xmerl data in them by default too.Wherever possible use Entity.update/2 to make changes, do not write to the Entity struct directly. If you must write directly you can use Entity.update/1 to resync the state of the record.","ref":"Smee.Entity.html","title":"Smee.Entity","type":"module"},{"doc":"Returns an entity that contains parsed XML data, greatly increasing its size but possibly making future processing faster.","ref":"Smee.Entity.html#bulkup/1","title":"Smee.Entity.bulkup/1","type":"function"},{"doc":"Returns a compressed entity, containing gzipped XML. This greatly reduces the size of the entity record.","ref":"Smee.Entity.html#compress/1","title":"Smee.Entity.compress/1","type":"function"},{"doc":"Returns true if the XML data in an entity has been compressed","ref":"Smee.Entity.html#compressed?/1","title":"Smee.Entity.compressed?/1","type":"function"},{"doc":"Returns a decompressed entity, with plain-text XML data. This makes the struct much larger.","ref":"Smee.Entity.html#decompress/1","title":"Smee.Entity.decompress/1","type":"function"},{"doc":"Returns a new %Entity{} struct if passed XML data for an entity and parent %Metadata{}.Defaults values are set using the parent metadata where possible.You can set or override various parts of the struct by passing options:md_uri - a URI that identifies a parentdownloaded_at - A DateTime to record when the record was downloadedmodified_at - A DateTime to record when the record was updated upstreamvalid_until - A DateTime to indicate when an entity expirespriority - An integer between 0 and 9 to show prioritytrustiness - a Float between 0.0 and 0.9 to indicate, well, trustiness.You won't normally need to do this yourself as entities can be extracted from Smee.Metadata.","ref":"Smee.Entity.html#derive/3","title":"Smee.Entity.derive/3","type":"function"},{"doc":"Returns a suggested filename for the entity.","ref":"Smee.Entity.html#filename/1","title":"Smee.Entity.filename/1","type":"function"},{"doc":"Returns a suggested filename for the entity in the specified format.Two formats can be specified: :sha1 and :uri","ref":"Smee.Entity.html#filename/2","title":"Smee.Entity.filename/2","type":"function"},{"doc":"Returns true if the entity has an IdP role.An entity may have more than one role.","ref":"Smee.Entity.html#idp?/1","title":"Smee.Entity.idp?/1","type":"function"},{"doc":"Returns a new %Entity{} struct if passed XML data.You can set or override various parts of the struct by passing options:md_uri - a URI that identifies a parentdownloaded_at - A DateTime to record when the record was downloadedmodified_at - A DateTime to record when the record was updated upstreamvalid_until - A DateTime to indicate when an entity expirespriority - An integer between 0 and 9 to show prioritytrustiness - a Float between 0.0 and 0.9 to indicate, well, trustiness.You won't normally need to do this yourself as entities can be extracted from Smee.Metadata.","ref":"Smee.Entity.html#new/2","title":"Smee.Entity.new/2","type":"function"},{"doc":"Returns the priority of the entity as a value between 0 and 9","ref":"Smee.Entity.html#priority/1","title":"Smee.Entity.priority/1","type":"function"},{"doc":"Returns an entity with parsed XML data removed, greatly reducing its size but possibly making future processing slower.","ref":"Smee.Entity.html#slim/1","title":"Smee.Entity.slim/1","type":"function"},{"doc":"Returns true if the entity has an SP role.An entity may have more than one role.","ref":"Smee.Entity.html#sp?/1","title":"Smee.Entity.sp?/1","type":"function"},{"doc":"Returns the trustiness level of the entity as a float between 0.0 and 0.9.","ref":"Smee.Entity.html#trustiness/1","title":"Smee.Entity.trustiness/1","type":"function"},{"doc":"Resyncs the internal state of an %Entity{} structIf changes have been made using Entity.update/2 then this will not be needed - it's there for when the struct has been changed directly","ref":"Smee.Entity.html#update/1","title":"Smee.Entity.update/1","type":"function"},{"doc":"Returns an updates %Entity{} struct with new XML, refreshing various parts of the struct correctly.This should be the only way updated Entities are produced - the raw struct should not be changed directly.","ref":"Smee.Entity.html#update/2","title":"Smee.Entity.update/2","type":"function"},{"doc":"Returns a parsed Erlang xmerl structure representing the entities XML, for use with xmerl, SweetXML and other tools.","ref":"Smee.Entity.html#xdoc/1","title":"Smee.Entity.xdoc/1","type":"function"},{"doc":"Returns the plain-text XML of the entity, whether or not it has been compressed.","ref":"Smee.Entity.html#xml/1","title":"Smee.Entity.xml/1","type":"function"},{"doc":"","ref":"Smee.Entity.html#t:t/0","title":"Smee.Entity.t/0","type":"type"},{"doc":"Processes %Metadata{} structs to extract various information, usually using XSLT.This module is intended to make common and relatively simple actions on potentially large XML files more efficiently than processing each %Entity{} struct in turn, and may rely on external tools such as xsltproc.These functions are useful in quick reports or to speed-up the internal workings of other modules.","ref":"Smee.Extract.html","title":"Smee.Extract","type":"module"},{"doc":"Returns a single %Entity{} struct extracted from the metadata, if present. Raises an exception if not present.","ref":"Smee.Extract.html#entity!/2","title":"Smee.Extract.entity!/2","type":"function"},{"doc":"Returns a map of all unique metadata Entity Attributes and their values","ref":"Smee.Extract.html#list_entity_attrs/1","title":"Smee.Extract.list_entity_attrs/1","type":"function"},{"doc":"Returns a list of all entity IDs in a Metadata struct.","ref":"Smee.Extract.html#list_ids/1","title":"Smee.Extract.list_ids/1","type":"function"},{"doc":"Returns a list of maps containing MDUI information for each entity in the metadata file.","ref":"Smee.Extract.html#mdui_info/1","title":"Smee.Extract.mdui_info/1","type":"function"},{"doc":"Downloads or loads the metadata specified in a %Smee.Source{} struct and returns a %Smee.Metadata{} struct.The fetch!/2 and fetch/2 functions can handle both remote and local sources, but sometimes it can be reassuring to prevent unexpected behaviour (perhaps if using a source defined by a user) so Smee.Fetch also contains local!/2 and remote!/2 which will each only accept certain types of sources.This module will do the work of creating suitable Metadata structs for you, so you should not normally need to create Metadata structs directly yourself.","ref":"Smee.Fetch.html","title":"Smee.Fetch","type":"module"},{"doc":"Uses the passed Source struct to load or download the requested metadata XML, and returns a Metadata struct containing the XML.Works with all types of Source, even MDQ services.Will raise an exception on any errors.","ref":"Smee.Fetch.html#fetch!/2","title":"Smee.Fetch.fetch!/2","type":"function"},{"doc":"Uses the passed Source struct to load the requested metadata XML, and returns a Metadata struct containing the XML Works with local Sources including MDQ services but will not accept metadata at a remote URL. Will raise an exception on any errors.","ref":"Smee.Fetch.html#local!/2","title":"Smee.Fetch.local!/2","type":"function"},{"doc":"Uses the passed Source struct to download the requested metadata XML, and returns a Metadata struct containing the XML in an :ok/:error tuple. Works with remote Sources including MDQ services but will not accept metadata in a local file.","ref":"Smee.Fetch.html#remote/2","title":"Smee.Fetch.remote/2","type":"function"},{"doc":"Uses the passed Source struct to download the requested metadata XML, and returns a Metadata struct containing the XML Works with remote Sources including MDQ services but will not accept metadata in a local file. Will raise an exception on any errors.","ref":"Smee.Fetch.html#remote!/2","title":"Smee.Fetch.remote!/2","type":"function"},{"doc":"Process a stream of entities to include or exclude entity structs matching the specified criteria.These functions are intended to be used with streams but should also work with simple lists too - but using lists to process larger metadata files is strongly discouraged.By default these functions include matching entities and exclude those that do not match, but this an be reversed. By default Smee.Filter.idp/3 will exclude entities that are no IdPs. But by specifying false as the third parameter the filter will be inverted and exclude entities that have an IdP role.","ref":"Smee.Filter.html","title":"Smee.Filter","type":"module"},{"doc":"Filters a stream of entities to include or exclude those that have an IdP role.The filter is positive by default but can be inverted by specifying false","ref":"Smee.Filter.html#idp/2","title":"Smee.Filter.idp/2","type":"function"},{"doc":"Filters a stream of entities to include or exclude those that have an SP role.The filter is positive by default but can be inverted by specifying false","ref":"Smee.Filter.html#sp/2","title":"Smee.Filter.sp/2","type":"function"},{"doc":"Filters a stream of entities to include or exclude those that have a trustiness equal or less than the specified number.Trustiness values are between 0.0 and 0.9.The filter is positive by default but can be inverted by specifying false","ref":"Smee.Filter.html#trustiness/3","title":"Smee.Filter.trustiness/3","type":"function"},{"doc":"Filters a stream of entities to include or exclude those that have one of the specified IDs.The filter is positive by default but can be inverted by specifying false","ref":"Smee.Filter.html#uri/3","title":"Smee.Filter.uri/3","type":"function"},{"doc":"Lint runs basic XML quality checks against XML strings","ref":"Smee.Lint.html","title":"Smee.Lint","type":"module"},{"doc":"Validates XML - checks that it is well-formed and complies with SAML metadata schema. It does *not check signatures or expiry.","ref":"Smee.Lint.html#validate/2","title":"Smee.Lint.validate/2","type":"function"},{"doc":"Smee.MDQ provides a client API to MDQ services, and also attempts to emulate MDQ-style behaviour with old-fashioned metadata aggregates. MDQ allows individual entities to be looked up when needed without downloading and processing an increasingly enormous metadata aggregate file.Emulation of MDQ for aggregates depends on caching and is usually slower for individual entity lookups even after the initial download and caching is complete.","ref":"Smee.MDQ.html","title":"Smee.MDQ","type":"module"},{"doc":"Returns the aggregated XML for the MDQ service as a Metadata struct, if one is available.","ref":"Smee.MDQ.html#aggregate!/1","title":"Smee.MDQ.aggregate!/1","type":"function"},{"doc":"Fetches an individual entity's metadata from the MDQ service and returns it as an Entity struct.Get attempts to behave like Ecto's Repo.get - it will return an entity or nil if the entity is unavailable.","ref":"Smee.MDQ.html#get/2","title":"Smee.MDQ.get/2","type":"function"},{"doc":"Fetches an individual entity's metadata from the MDQ service and returns it as an Entity struct.Get attempts to behave like Ecto's Repo.get! - it will return an entity or raises an exception if the entity is unavailable.This is identical to lookup!2 but exists for consistency.","ref":"Smee.MDQ.html#get!/2","title":"Smee.MDQ.get!/2","type":"function"},{"doc":"Returns a list of all entity ID URIs available at the MDQ service.This will probably involve downloading the MDQ service's aggregate in most cases, but this aggregate will be cached.","ref":"Smee.MDQ.html#list!/1","title":"Smee.MDQ.list!/1","type":"function"},{"doc":"Fetches an individual entity's metadata from the MDQ service and returns it as an Entity struct in an :ok/:error tuple.Missing or unknown entities will cause an {:error, :http_404} result.","ref":"Smee.MDQ.html#lookup/2","title":"Smee.MDQ.lookup/2","type":"function"},{"doc":"Fetches an individual entity's metadata from the MDQ service and returns it as an Entity struct.This is identical to get!2 but exists for consistency.","ref":"Smee.MDQ.html#lookup!/2","title":"Smee.MDQ.lookup!/2","type":"function"},{"doc":"A convenient shortcut for specifying an MDQ service as a source.This is functionally identical to using Smee.Source.new(url, type: :mdq)","ref":"Smee.MDQ.html#source/2","title":"Smee.MDQ.source/2","type":"function"},{"doc":"Fetches every entity from the MDQ service one by one and returns them as a stream of Entity structs.This stream allows entities to be processed individually, using relatively little memory.The stream is relatively laid-back even on a fast M1 Mac, but please don't abuse this function and overwhelm public MDQ services.","ref":"Smee.MDQ.html#stream/1","title":"Smee.MDQ.stream/1","type":"function"},{"doc":"Fetches the specified entities from the MDQ service one by one and returns them as a stream of Entity structs.This stream allows entities to be processed individually, using relatively little memory.The stream is relatively laid-back even on a fast M1 Mac, but please don't abuse this function and overwhelm public MDQ services.","ref":"Smee.MDQ.html#stream/2","title":"Smee.MDQ.stream/2","type":"function"},{"doc":"If passed an entity ID URI is returns the MDQ &quot;transformed&quot; version of the identifer, based on a sha1 hash.Already-transformed identifiers are passed through unchanged.","ref":"Smee.MDQ.html#transform_uri/1","title":"Smee.MDQ.transform_uri/1","type":"function"},{"doc":"Returns the full download URL for an entity at the specified service.","ref":"Smee.MDQ.html#url/2","title":"Smee.MDQ.url/2","type":"function"},{"doc":"The Metadata module wraps up Metadata XML into a struct and contains functions that may be helpful when working with them. The metadata is either an aggregate (as used by federations to contain many entity records) or a single entity. Many of the functions mirror those in the Smee.Entity module - the same actions but on larger source XML rather than on fragments.The XML in metadata structs can be compressed or decompressed, but unlike Entities there is no cached, parsed xmlerl record by default - this is to save time and memory. Wherever possible use Metadata.update/2 to make changes, do not write to the Entity struct directly. If you must write directly you can use Metadata.update/1 to resync the state of the record.Methods in Smee.Metadata can be used to extract individual entity records each containing a fragment of XML. It's strongly recommended to stream these using stream_entities2 to save on memory, or select a particular entity using entity2.","ref":"Smee.Metadata.html","title":"Smee.Metadata","type":"module"},{"doc":"Returns compressed metadata, containing gzipped XML. This greatly reduces the size of the metadata record.","ref":"Smee.Metadata.html#compress/1","title":"Smee.Metadata.compress/1","type":"function"},{"doc":"Returns true if the XML data in an metadata struct has been compressed","ref":"Smee.Metadata.html#compressed?/1","title":"Smee.Metadata.compressed?/1","type":"function"},{"doc":"Returns the number of entities in the metadata file","ref":"Smee.Metadata.html#count/1","title":"Smee.Metadata.count/1","type":"function"},{"doc":"Returns decompressed metadata, with plain-text XML data. This makes the struct much larger.","ref":"Smee.Metadata.html#decompress/1","title":"Smee.Metadata.decompress/1","type":"function"},{"doc":"Returns a new metadata struct based on the streamed entities passed as the first parameter.You can set or override various parts of the struct by passing options:url - the original location of the metadatauri - a URI that identifies the metadata (Name)downloaded_at - A DateTime to record when the record was downloadedmodified_at - A DateTime to record when the record was updated upstreamvalid_until - A DateTime to indicate when an entity expirespriority - An integer between 0 and 9 to show prioritytrustiness - a Float between 0.0 and 0.9 to indicate, well, trustiness.etag - a string to use as an etag (unique content identifier).cert_url - location of a certificate to use for signature verificationcert_fingerprint - fingerprint of the certificate to use for certificate verificationlabel - a description for the metadata","ref":"Smee.Metadata.html#derive/2","title":"Smee.Metadata.derive/2","type":"function"},{"doc":"Returns all entities in the metadata as a list of entity structs.This can produce very large lists very slowly. The stream_entities2 function is much better.","ref":"Smee.Metadata.html#entities/1","title":"Smee.Metadata.entities/1","type":"function"},{"doc":"Returns the specified entity from the metadata in an :ok/:error struct","ref":"Smee.Metadata.html#entity/2","title":"Smee.Metadata.entity/2","type":"function"},{"doc":"Returns the specified entity from the metadata or raises an exception if not found","ref":"Smee.Metadata.html#entity!/2","title":"Smee.Metadata.entity!/2","type":"function"},{"doc":"Returns a list of all entity IDs in the metadata","ref":"Smee.Metadata.html#entity_ids/1","title":"Smee.Metadata.entity_ids/1","type":"function"},{"doc":"Returns a suggested filename for the metadata.","ref":"Smee.Metadata.html#filename/1","title":"Smee.Metadata.filename/1","type":"function"},{"doc":"Returns a suggested filename for the metadata in the specified format.Two formats can be specified: :sha1 and :uri","ref":"Smee.Metadata.html#filename/2","title":"Smee.Metadata.filename/2","type":"function"},{"doc":"Returns a new metadata struct based on the XML data passed as the first parameter.You can set or override various parts of the struct by passing options:url - the original location of the metadatauri - a URI that identifies the metadata (Name)downloaded_at - A DateTime to record when the record was downloadedmodified_at - A DateTime to record when the record was updated upstreamvalid_until - A DateTime to indicate when an entity expirespriority - An integer between 0 and 9 to show prioritytrustiness - a Float between 0.0 and 0.9 to indicate, well, trustiness.etag - a string to use as an etag (unique content identifier).cert_url - location of a certificate to use for signature verificationcert_fingerprint - fingerprint of the certificate to use for certificate verificationlabel - a description for the metadataIn most cases it is better to use Smee.Source and then Smee.Fetch to generate a metadata struct.","ref":"Smee.Metadata.html#new/2","title":"Smee.Metadata.new/2","type":"function"},{"doc":"Returns one randomly selected entity from the metadata","ref":"Smee.Metadata.html#random_entity/1","title":"Smee.Metadata.random_entity/1","type":"function"},{"doc":"Returns a stream of all entities in the metadata.","ref":"Smee.Metadata.html#stream_entities/2","title":"Smee.Metadata.stream_entities/2","type":"function"},{"doc":"Resyncs the internal state of a %Metadata{} structIf changes have been made using Metadata.update/2 then this will not be needed - it's there for when the struct has been changed directly","ref":"Smee.Metadata.html#update/1","title":"Smee.Metadata.update/1","type":"function"},{"doc":"Returns an updates %Metadata{} struct with new XML, refreshing various parts of the struct correctly.This should be the only way updated Metadata structs are produced - the raw struct should not be changed directly.","ref":"Smee.Metadata.html#update/2","title":"Smee.Metadata.update/2","type":"function"},{"doc":"Returns a parsed Erlang xmerl structure representing the metadata XML, for use with xmerl, SweetXML and other tools.","ref":"Smee.Metadata.html#xml/1","title":"Smee.Metadata.xml/1","type":"function"},{"doc":"","ref":"Smee.Metadata.html#t:t/0","title":"Smee.Metadata.t/0","type":"type"},{"doc":"Publish exports streams or lists of entity structs into various formats.At present the output formats are SAML XML (individual and collections) and simple index text files. Formats can be output as binary strings or streamed. Streamed output can be useful for web services, allowing gradual downloads generated on-the-fly with no need to render a very large document in advance.","ref":"Smee.Publish.html","title":"Smee.Publish","type":"module"},{"doc":"","ref":"Smee.Publish.html#aggregate_footer/1","title":"Smee.Publish.aggregate_footer/1","type":"function"},{"doc":"","ref":"Smee.Publish.html#aggregate_header/1","title":"Smee.Publish.aggregate_header/1","type":"function"},{"doc":"Returns an index text document","ref":"Smee.Publish.html#to_index/2","title":"Smee.Publish.to_index/2","type":"function"},{"doc":"Returns a streamed index file, a plain text list of entity IDs.","ref":"Smee.Publish.html#to_index_stream/2","title":"Smee.Publish.to_index_stream/2","type":"function"},{"doc":"Returns the estimated size of a streamed index file without generating it in advance.","ref":"Smee.Publish.html#to_index_stream_size/2","title":"Smee.Publish.to_index_stream_size/2","type":"function"},{"doc":"Returns a SAML metadata XML file, potentially very large.","ref":"Smee.Publish.html#to_xml/2","title":"Smee.Publish.to_xml/2","type":"function"},{"doc":"Returns a streamed SAML metadata XML file","ref":"Smee.Publish.html#to_xml_stream/2","title":"Smee.Publish.to_xml_stream/2","type":"function"},{"doc":"Returns the estimated size of a streamed SAML metadata XML file without generating it in advance.","ref":"Smee.Publish.html#to_xml_stream_size/2","title":"Smee.Publish.to_xml_stream_size/2","type":"function"},{"doc":"X","ref":"Smee.Security.html","title":"Smee.Security","type":"module"},{"doc":"","ref":"Smee.Security.html#verify/1","title":"Smee.Security.verify/1","type":"function"},{"doc":"","ref":"Smee.Security.html#verify!/1","title":"Smee.Security.verify!/1","type":"function"},{"doc":"","ref":"Smee.Security.html#verify?/1","title":"Smee.Security.verify?/1","type":"function"},{"doc":"Defines sources for metadata, which can then be Fetched and prodoce Metadata structs. Source structs are the usual starting-place for Smee tasks. Source structs act as the configuration for metadata creation. They require a URL but various other options can be set and their effect will propagate down to Entity records.","ref":"Smee.Source.html","title":"Smee.Source","type":"module"},{"doc":"Attempts to validate a source struct, and will return an :ok/:error tuple containing the Source if it passes checks.","ref":"Smee.Source.html#check/2","title":"Smee.Source.check/2","type":"function"},{"doc":"Attempts to validate a source struct, and will return the Source if it passes checks, or raise an exception.","ref":"Smee.Source.html#check!/2","title":"Smee.Source.check!/2","type":"function"},{"doc":"Creates a new source struct, describing where and how to find metadata.The only essential parameter is a URL for the metadata. URLs can have file:, http://, or https:// schemes. Bare filesystem paths can also be passed and will be converted to file: URLs.MDQ service clients should be configured with the base URL of the service, not the /entity/ endpoint, and a type of :mdq. Alternatively the Smee.MDQ.source/2 function can be used as a shortcut.Available options include:type: determines the general type of metadata - :aggregate (the default), :single and :mdqcert_url: location of certificate to use when verifying signed metadatacert_fingerprint: SHA1 fingerprint of the signing certificatecache: should HTTP caching be enabled for downloads. true or false, defaults to true.redirects: maximum number of 302 redirects to followretries: number of retries to attemptlabel: a relatively useless label for the metadatapriority: integer between 0 and 9, used for comparing metadatatrustiness: float between 0.0 and 0.9, for comparing metadatastrict: defaults to false. If enabled some stricter checks are enabledMDQ sources are intended for use with the Smee.MDQ API but will also support normal fetch requests.","ref":"Smee.Source.html#new/2","title":"Smee.Source.new/2","type":"function"},{"doc":"","ref":"Smee.Source.html#t:t/0","title":"Smee.Source.t/0","type":"type"},{"doc":"A collection of simple statistics tools that use streams of entity structs (and maybe lists)","ref":"Smee.Stats.html","title":"Smee.Stats","type":"module"},{"doc":"Counts the number of entities in the stream and returns a value (runs and ends the stream)Is this a placeholder in an almost empty module? Absolutely.","ref":"Smee.Stats.html#count/1","title":"Smee.Stats.count/1","type":"function"},{"doc":"Tools for manipulating metadata XML.This module is intended to make common and relatively simple actions on potentially large XML files more efficiently than processing a metadata or entities using xmerl, and may rely on external tools such as xsltproc.These functions are useful for pre-processing metadata XML before it's broken up into entity struct, and to speed-up the internal workings of other modules.","ref":"Smee.Transform.html","title":"Smee.Transform","type":"module"},{"doc":"Strips various extraneous parts from the metadata and returns a new struct in an :ok/:error tuple","ref":"Smee.Transform.html#decruft_idp/1","title":"Smee.Transform.decruft_idp/1","type":"function"},{"doc":"Strips various extraneous parts from the metadata and returns a new struct in an :ok/:error tuple","ref":"Smee.Transform.html#decruft_sp/1","title":"Smee.Transform.decruft_sp/1","type":"function"},{"doc":"Returns a metadata struct with all comments removed, in an :ok/:error struct","ref":"Smee.Transform.html#strip_comments/1","title":"Smee.Transform.strip_comments/1","type":"function"},{"doc":"Applies an XSLT stylesheet to a metadata struct, returning a transformed metadata struct in an :ok/error tuple","ref":"Smee.Transform.html#transform/3","title":"Smee.Transform.transform/3","type":"function"},{"doc":"Returns a metadata struct with the validUntil date updated, in an :ok/:error struct","ref":"Smee.Transform.html#valid_until/2","title":"Smee.Transform.valid_until/2","type":"function"},{"doc":"Returns a metadata struct with the validUntil date updated, or raises if an error occurs","ref":"Smee.Transform.html#valid_until!/2","title":"Smee.Transform.valid_until!/2","type":"function"},{"doc":"SmeeSmee is a pragmatic library for handling SAML metadata with Elixir, Erlang or any other BEAM language.Smee started life as an sprawling mess of code used over five years in an internal application to download and process SAML metadata. It's now been extracted, restructured and made a little less messy, with the aim of reusing it in other projects. Smee can be used within applications or in simpler scripts for processing metadata.","ref":"readme.html","title":"Smee","type":"extras"},{"doc":"Download remote SAML metadata or load local files, with effective cachingManage and compare metadata files and individual entity metadata MDQ API (which can also emulate MDQ style lookups with aggregate files)A focus on streaming with reliable and surprisingly low memory usageFilter entity streams by various criteriaValidate XML signatures, automatically download and confirm signing certificatesTransform metadata using XSLT, or extract dataAccess XML using Erlang's Xmerl library (sweetened by SweetXML)Recombine entity streams into aggregates or other data formatsCan be used with applications or in simple .exs scriptsThe top level Smee module contains simplified, top level functions better suited to simpler scripts. Other modules in Smee contain more tools for handling SAML metadata, such as:Smee.Source - define sources of metadataSmee.Metadata - functions for handling metadata aggregatesSmee.Entity - individual SAML entity definitionsSmee.Extract - processing metadata to extract informationSmee.Fetch - downloading metadata sourcesSmee.MDQ - functions for MDQ clients (and emulating MDQ clients)Smee.Filter - filtering streams of entity recordsSmee.Transform - processing and editing entity XMLSmee.Publish - Formatting and outputting metadata in various formatsSmee.Stats - Simple stats for entity streamsSmee.Lint - XML validation and reformatting","ref":"readme.html#features","title":"Smee - Features","type":"extras"},{"doc":"Getting entity details from an MDQ servicealias Smee.MDQ cern_idp = MDQ.source(&quot;http://mdq.ukfederation.org.uk/&quot;) |&gt; MDQ.lookup!(&quot;https://cern.ch/login&quot;) Downloading and streaming a metadata aggregate to output a list of all IdP entityIDsalias Smee.{Source, Fetch, Filter, Metadata} &quot;http://metadata.ukfederation.org.uk/ukfederation-metadata.xml&quot; |&gt; Source.new() |&gt; Fetch.remote!() |&gt; Metadata.stream_entities() |&gt; Filter.idp() |&gt; Stream.map(fn e -&gt; e.uri end) |&gt; Enum.to_list() Create an aggregate that only contains SPsalias Smee.{Source, Fetch, Filter, Metadata, Publish} &quot;http://metadata.ukfederation.org.uk/ukfederation-metadata.xml&quot; |&gt; Source.new() |&gt; Fetch.remote!() |&gt; Metadata.stream_entities() |&gt; Filter.sp() |&gt; Stream.map(fn e -&gt; e.uri end) |&gt; Publish.to_xml() Munge XML with XSLTalias Smee.{Source, Fetch, Transform, Metadata} ## Using a builtin function new_xml = Source.new(&quot;adfs.xml&quot;) |&gt; Fetch.local!() |&gt; Transform.decruft_sp!() |&gt; Metadata.xml() File.save!(&quot;adfs_new.xml&quot;, new_xml) ## Or any XSLT stylesheet {:ok, updated_metadata} = Transform.transform(metadata, stylesheet, [exampleParam: &quot;example value&quot;])","ref":"readme.html#examples","title":"Smee - Examples","type":"extras"},{"doc":"If available in Hex, the package can be installed by adding Smee to your list of dependencies in mix.exs:def deps do [ {:Smee, &quot;~&gt; 0.1.0&quot;} ] endPlease note: Smee does not do all processing itself using Elixir - it sometimes cheats (OK, it often cheats) by sending data to external programs for processing. At the moment it requires the following commandline utilities:xmlsec1xmllintxsltprocOn a Mac with Homebrew: brew install xmlsec1 libxml2 libxslt (Apple's xsltproc is probably already installed)On Debian Linux: apt-get install xmlsec1 libxml2-utils xsltprocA future version of Smee will support alternative sets of backends.","ref":"readme.html#installation","title":"Smee - Installation","type":"extras"},{"doc":"Metadata Database: Smee was extracted from an application that downloads, processes and stores SAML metadata in a database for access with an API. The original code had become rather messy and was very difficult to test - hopefully Smee will be an improvement and will be used to replace the original code. Smee doesn't currently handle digestion and storage.MDQ Service: We've made a streaming MDQ service using Smee that should soon be available Reports: We've generated reports for clients using Smee","ref":"readme.html#uses","title":"Smee - Uses","type":"extras"},{"doc":"These aren't quite the same but are mature, widely used projects that are a lot more stable than Smee. In many cases you should definitely use these instead of Smee.SAML Library by Tom Scavo A collection of Bash and XSLT scripts PyFF A metadata pipeline and server using Python and YAMLShibboleth Metadata Aggregator Widely used Java metadata pipeline","ref":"readme.html#possible-alternatives","title":"Smee - Possible Alternatives","type":"extras"},{"doc":"SAML MetadataWikipedia's summaryMDQUK Federation Explains MDQUnusual types of SAML metadataOauth/OIDCADFS does it's own thingCAS!","ref":"readme.html#references","title":"Smee - References","type":"extras"},{"doc":"Various Github repositories have been plundered in a search for XSLT stylesheets, in particular:SAML Library by Tom ScavoSWAMID FederationUK Access Management Federation's metadata processing pipelineThe use of the Apache2 license in the above projects led to the Apache2 license for Smee.Other sources and inspirations:The XSLT package for Elixir","ref":"readme.html#contributors-and-sources","title":"Smee - Contributors and Sources","type":"extras"},{"doc":"Documentation can be generated with ExDoc and published on HexDocs. Once published, the docs can be found at https://hexdocs.pm/smee.","ref":"readme.html#documentation","title":"Smee - Documentation","type":"extras"},{"doc":"You can request new features by creating an issue, or submit a pull request with your contribution.","ref":"readme.html#contributing","title":"Smee - Contributing","type":"extras"},{"doc":"Copyright (c) 2023 Digital Identity Ltd, UKSmee is Apache 2.0 licensed.","ref":"readme.html#copyright-and-license","title":"Smee - Copyright and License","type":"extras"},{"doc":"Smee is not endorsed by The Shibboleth Foundation or any of the NREN's that may have had their XSLT borrowed. The API will definitely change considerably in the first few releases after 0.1.0 - it is not stable!","ref":"readme.html#disclaimer","title":"Smee - Disclaimer","type":"extras"},{"doc":"Apache License Version 2.0, January 2004 http://www.apache.org/licenses/ TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION 1. Definitions. &quot;License&quot; shall mean the terms and conditions for use, reproduction, and distribution as defined by Sections 1 through 9 of this document. &quot;Licensor&quot; shall mean the copyright owner or entity authorized by the copyright owner that is granting the License. &quot;Legal Entity&quot; shall mean the union of the acting entity and all other entities that control, are controlled by, or are under common control with that entity. For the purposes of this definition, &quot;control&quot; means (i) the power, direct or indirect, to cause the direction or management of such entity, whether by contract or otherwise, or (ii) ownership of fifty percent (50%) or more of the outstanding shares, or (iii) beneficial ownership of such entity. &quot;You&quot; (or &quot;Your&quot;) shall mean an individual or Legal Entity exercising permissions granted by this License. &quot;Source&quot; form shall mean the preferred form for making modifications, including but not limited to software source code, documentation source, and configuration files. &quot;Object&quot; form shall mean any form resulting from mechanical transformation or translation of a Source form, including but not limited to compiled object code, generated documentation, and conversions to other media types. &quot;Work&quot; shall mean the work of authorship, whether in Source or Object form, made available under the License, as indicated by a copyright notice that is included in or attached to the work (an example is provided in the Appendix below). &quot;Derivative Works&quot; shall mean any work, whether in Source or Object form, that is based on (or derived from) the Work and for which the editorial revisions, annotations, elaborations, or other modifications represent, as a whole, an original work of authorship. For the purposes of this License, Derivative Works shall not include works that remain separable from, or merely link (or bind by name) to the interfaces of, the Work and Derivative Works thereof. &quot;Contribution&quot; shall mean any work of authorship, including the original version of the Work and any modifications or additions to that Work or Derivative Works thereof, that is intentionally submitted to Licensor for inclusion in the Work by the copyright owner or by an individual or Legal Entity authorized to submit on behalf of the copyright owner. For the purposes of this definition, &quot;submitted&quot; means any form of electronic, verbal, or written communication sent to the Licensor or its representatives, including but not limited to communication on electronic mailing lists, source code control systems, and issue tracking systems that are managed by, or on behalf of, the Licensor for the purpose of discussing and improving the Work, but excluding communication that is conspicuously marked or otherwise designated in writing by the copyright owner as &quot;Not a Contribution.&quot; &quot;Contributor&quot; shall mean Licensor and any individual or Legal Entity on behalf of whom a Contribution has been received by Licensor and subsequently incorporated within the Work. 2. Grant of Copyright License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable copyright license to reproduce, prepare Derivative Works of, publicly display, publicly perform, sublicense, and distribute the Work and such Derivative Works in Source or Object form. 3. Grant of Patent License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable (except as stated in this section) patent license to make, have made, use, offer to sell, sell, import, and otherwise transfer the Work, where such license applies only to those patent claims licensable by such Contributor that are necessarily infringed by their Contribution(s) alone or by combination of their Contribution(s) with the Work to which such Contribution(s) was submitted. If You institute patent litigation against any entity (including a cross-claim or counterclaim in a lawsuit) alleging that the Work or a Contribution incorporated within the Work constitutes direct or contributory patent infringement, then any patent licenses granted to You under this License for that Work shall terminate as of the date such litigation is filed. 4. Redistribution. You may reproduce and distribute copies of the Work or Derivative Works thereof in any medium, with or without modifications, and in Source or Object form, provided that You meet the following conditions: (a) You must give any other recipients of the Work or Derivative Works a copy of this License; and (b) You must cause any modified files to carry prominent notices stating that You changed the files; and (c) You must retain, in the Source form of any Derivative Works that You distribute, all copyright, patent, trademark, and attribution notices from the Source form of the Work, excluding those notices that do not pertain to any part of the Derivative Works; and (d) If the Work includes a &quot;NOTICE&quot; text file as part of its distribution, then any Derivative Works that You distribute must include a readable copy of the attribution notices contained within such NOTICE file, excluding those notices that do not pertain to any part of the Derivative Works, in at least one of the following places: within a NOTICE text file distributed as part of the Derivative Works; within the Source form or documentation, if provided along with the Derivative Works; or, within a display generated by the Derivative Works, if and wherever such third-party notices normally appear. The contents of the NOTICE file are for informational purposes only and do not modify the License. You may add Your own attribution notices within Derivative Works that You distribute, alongside or as an addendum to the NOTICE text from the Work, provided that such additional attribution notices cannot be construed as modifying the License. You may add Your own copyright statement to Your modifications and may provide additional or different license terms and conditions for use, reproduction, or distribution of Your modifications, or for any such Derivative Works as a whole, provided Your use, reproduction, and distribution of the Work otherwise complies with the conditions stated in this License. 5. Submission of Contributions. Unless You explicitly state otherwise, any Contribution intentionally submitted for inclusion in the Work by You to the Licensor shall be under the terms and conditions of this License, without any additional terms or conditions. Notwithstanding the above, nothing herein shall supersede or modify the terms of any separate license agreement you may have executed with Licensor regarding such Contributions. 6. Trademarks. This License does not grant permission to use the trade names, trademarks, service marks, or product names of the Licensor, except as required for reasonable and customary use in describing the origin of the Work and reproducing the content of the NOTICE file. 7. Disclaimer of Warranty. Unless required by applicable law or agreed to in writing, Licensor provides the Work (and each Contributor provides its Contributions) on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied, including, without limitation, any warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are solely responsible for determining the appropriateness of using or redistributing the Work and assume any risks associated with Your exercise of permissions under this License. 8. Limitation of Liability. In no event and under no legal theory, whether in tort (including negligence), contract, or otherwise, unless required by applicable law (such as deliberate and grossly negligent acts) or agreed to in writing, shall any Contributor be liable to You for damages, including any direct, indirect, special, incidental, or consequential damages of any character arising as a result of this License or out of the use or inability to use the Work (including but not limited to damages for loss of goodwill, work stoppage, computer failure or malfunction, or any and all other commercial damages or losses), even if such Contributor has been advised of the possibility of such damages. 9. Accepting Warranty or Additional Liability. While redistributing the Work or Derivative Works thereof, You may choose to offer, and charge a fee for, acceptance of support, warranty, indemnity, or other liability obligations and/or rights consistent with this License. However, in accepting such obligations, You may act only on Your own behalf and on Your sole responsibility, not on behalf of any other Contributor, and only if You agree to indemnify, defend, and hold each Contributor harmless for any liability incurred by, or claims asserted against, such Contributor by reason of your accepting any such warranty or additional liability. END OF TERMS AND CONDITIONS APPENDIX: How to apply the Apache License to your work. To apply the Apache License to your work, attach the following boilerplate notice, with the fields enclosed by brackets &quot;[]&quot; replaced with your own identifying information. (Don't include the brackets!) The text should be enclosed in the appropriate comment syntax for the file format. We also recommend that a file or class name and description of purpose be included on the same &quot;printed page&quot; as the copyright notice for easier identification within third-party archives. Copyright [yyyy] [name of copyright owner] Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.","ref":"license.html","title":"LICENSE","type":"extras"}]